import { HOST_FILE_HASH_MAP } from "@/constants/constants";
import { xor } from "lodash-es";

export function isElectron() {
  return typeof window.nodeRequire != "undefined";
}

const fs: typeof import("fs") = isElectron() && window.nodeRequire("fs");

/** 执行本地exe文件 */
export function execLocal(exeName: string): Promise<void> {
  if (isElectron()) {
    logger({
      pgu: "AUTO",
      cnl: ["local", "server"],
      dtl: "不在Electron中，调用 " + exeName + " 失败",
    });
    throw new Error("不在Electron中，调用 fs 失败");
  }
  return new Promise<void>((resolve) => {
    // eslint-disable-next-line
    window
      .nodeRequire("node-cmd")
      .get(
        "cmd /c chcp 65001>nul && " + exeName,
        async (err: Error, data: string, stderr: string) => {
          console.log(exeName, err, data); // 未免过多日志，此处后续可以关闭
          logger({
            pgu: "AUTO",
            cnl: ["console", "local", "server"],
            act: exeName + " called",
            ejn: JSON.stringify(err),
            dtl: data,
            ext: {
              stderr: JSON.stringify(stderr),
              absPath: exeName.includes(":"),
            },
          });
          // 如果相对路径没找到，则通过绝对路径来执行
          if (!exeName.includes(":") && err) {
            const remote = window.nodeRequire("electron").remote;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-call
            const fs: typeof import("fs") = remote.require("fs");

            // eslint-disable-next-line @typescript-eslint/no-unsafe-call
            const path: typeof import("path") = remote.require("path");

            const [exePath, exeParams] = exeName.split(" ");
            // eslint-disable-next-line @typescript-eslint/no-unsafe-call
            const absPath = path.join(
              // eslint-disable-next-line @typescript-eslint/no-unsafe-call
              remote.app.getAppPath() as string,
              "../../",
              exePath
            );
            if (fs.existsSync(absPath)) {
              try {
                await execLocal([absPath, exeParams].join(" ").trim());
              } catch (error) {
                console.log("second try error", absPath);
                logger({
                  cnl: ["local", "server"],
                  dtl: "second try error: " + absPath,
                });
              }
            }
          }
          resolve();
        }
      );
  });
}

/** 文件路径是否存在 */
export function fileExists(file: string): boolean {
  if (typeof window.nodeRequire == "undefined") {
    logger({
      pgu: "AUTO",
      cnl: ["local", "server"],
      dtl: "不在Electron中，调用 fs 失败",
    });
    throw new Error("不在Electron中，调用 fs 失败");
  }
  return fs.existsSync(file);
}

/** 检测当前运行的进程中是否有“向日葵” */
export function nodeCheckRemoteDesktop(): boolean {
  logger({
    pgu: "AUTO",
    cnl: ["local", "server"],
    key: "nodeCheckRemoteDesktop",
  });

  // eslint-disable-next-line @typescript-eslint/no-unsafe-call
  const appList: string = window
    .nodeRequire("child_process")
    .execSync("cmd /c chcp 65001>nul && tasklist /FO CSV")
    .toString();
  // console.debug(appList);

  const regex = new RegExp("sunloginclient|选择免安装运行，截图识别", "gi");
  return !!(appList?.match(regex) || []).length;
}

let previousAppList: string[] = [];
/** 将电脑运行的进程记录到日志 */
export function nodeCheckProcess(): void {
  try {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-call
    const appListCP: string = window
      .nodeRequire("child_process")
      .execSync("cmd /c chcp 65001>nul && tasklist /FO CSV")
      .toString();
    // 不能打印，electron-log 不能接收
    // console.log(appList);
    let appList = appListCP.split("\r\n");
    appList.shift();
    appList = appList.map((v) => v.split(",")[0]);
    const xorRes = xor(previousAppList, appList);
    if (xorRes && xorRes.length > 0) {
      logger({
        pgu: "AUTO",
        cnl: ["local", "server"],
        key: "nodeCheckProcess",
        dtl: xorRes.join(" ||| "),
      });
    }
    previousAppList = appList;
  } catch (error) {
    logger({
      cnl: ["console", "local", "server"],
      pgu: "AUTO",
      ejn: JSON.stringify(error),
    });
  }
}

/** 检测当前程序包的完整性。 */
export function checkMainExe(): boolean {
  try {
    let iid: string = window.nodeRequire("process").pid;
    const cp: typeof import("child_process") =
      window.nodeRequire("child_process");
    iid = cp
      .execSync(
        `cmd /c chcp 65001>nul && C:\\Windows\\System32\\wbem\\wmic process where ^(processid^=${iid}^) get parentprocessid /value`
      )
      .toString();
    iid = iid.replace("ParentProcessId=", "").trim();
    console.log(iid);
    logger({
      cnl: ["local", "console", "server"],
      key: "checkMainExe",
      dtl: `iid1: ${iid}`,
    });
    iid = cp
      .execSync(
        `cmd /c chcp 65001>nul && C:\\Windows\\System32\\wbem\\wmic process where ^(processid^=${iid}^) get parentprocessid /value`
      )
      .toString();
    iid = iid.replace("ParentProcessId=", "").trim();
    logger({
      cnl: ["local", "console", "server"],
      key: "checkMainExe",
      dtl: `iid2: ${iid}`,
    });

    const executablePathBuffer = cp.execSync(
      `cmd /c chcp 65001>nul && C:\\Windows\\System32\\wbem\\wmic process where ^(processid^=${iid}^) get executablepath /value`
    );
    console.log(executablePathBuffer);
    const encoding = window.nodeRequire("encoding");
    // eslint-disable-next-line @typescript-eslint/no-unsafe-call
    let executablePath: string = encoding
      .convert(executablePathBuffer, "utf8", "gbk")
      .toString();
    logger({
      cnl: ["local", "console", "server"],
      key: "checkMainExe",
      dtl: executablePath,
    });
    executablePath = executablePath
      .replace("ExecutablePath=", "")
      .trim()
      .replace(/&amp;/g, "&");
    if (executablePath === eval(`process.env.PORTABLE_EXECUTABLE_FILE`)) {
      const crypto: typeof import("crypto") = window.nodeRequire("crypto");
      const getHash = crypto
        .createHmac("sha256", "abcdefg")
        .update(fs.readFileSync(executablePath))
        .digest("hex");
      console.log("the hash: ", getHash);
      logger({
        cnl: ["local", "console", "server"],
        key: "checkMainExe",
        dtl: `the hash: ${getHash}`,
      });
      if (HOST_FILE_HASH_MAP.get(window.location.hostname) === getHash) {
        return true;
      }
    }
    // check filepath executablePath md5
  } catch (error) {
    console.log(error);
    logger({
      cnl: ["local", "console", "server"],
      key: "checkMainExe",
      ejn: JSON.stringify(error),
    });
  }
  return false;
}
